from bioservices import BioMart
import pytest


@pytest.fixture
def biomart():
    biomart = BioMart(host='www.ensembl.org', verbose=False)
    biomart.mart_test = 'ENSEMBL_MART_ENSEMBL'
    return biomart


@pytest.mark.flaky
def test_version(biomart):
    biomart.version(biomart.mart_test)


@pytest.mark.flaky
def _test_datasets(biomart):
    # there are about 70 datasets but let us check that at least the list is
    # not emptyt
    assert len(biomart.datasets(biomart.mart_test)) > 2

    assert "mmusculus_gene_ensembl" in biomart.datasets(biomart.mart_test)


@pytest.mark.flaky
def _test_attributes(biomart):
    assert 'oanatinus_gene_ensembl' in \
        biomart.valid_attributes[biomart.mart_test]


@pytest.mark.flaky
def test_filteres(biomart):
    biomart.filters("oanatinus_gene_ensembl")


@pytest.mark.flaky
def test_config(biomart):
    biomart.configuration("oanatinus_gene_ensembl")


#fails on travais sometines
@pytest.mark.flaky
def _test_query(biomart):
    res = biomart.query(biomart._xml_example)
    assert "ENSMUS" in res


@pytest.mark.flaky
def test_xml(biomart):
    # build own xml using the proper functions
    biomart.add_dataset_to_xml("mmusculus_gene_ensembl")
    biomart.get_xml()


@pytest.mark.flaky
def test_biomart_constructor():
    s = BioMart()
    try:
        s.registry()
    except:
        pass
    try:
        s.host = "dummy"
    except:
        pass
    s.host = "www.ensembl.org"

# # reactome not maintained anymore ?
# https://support.bioconductor.org/p/62622/
def _test_reactome_example():
    # this is not working anymore...
    s = BioMart("reactome.org")
    s.lookfor("reactome")
    s.datasets("REACTOME")
    #['interaction', 'complex', 'reaction', 'pathway']
    s.new_query()
    s.add_dataset_to_xml("pathway")
    s.add_filter_to_xml("species_selection", "Homo sapiens")
    s.add_attribute_to_xml("pathway_db_id")
    s.add_attribute_to_xml("_displayname")
    xmlq = s.get_xml()
    res = s.query(xmlq)




from bioservices import PDBe





def test_pdbe():
    p = PDBe(verbose=False)
    res = p.get_summary('1cbs')
    assert len(res)==1
    res = p.get_summary('1cbs,2kv8')
    assert len(res)==2
    res = p.get_summary(['1cbs', '2kv8'])
    assert len(res)==2

    p.get_molecules('1cbs')
    p.get_molecules('1cbs,2kv8')

    p.get_related_publications('1cbs')
    p.get_related_publications('1cbs')

    p.get_experiment('1cbs')
    p.get_experiment('1cbs,2kv8')

    p.get_nmr_resources('1cbs')
    p.get_nmr_resources('1cbs,2kv8')

    p.get_ligand_monomers('1cbs')
    p.get_ligand_monomers('1cbs,2kv8')

    p.get_modified_residues('4v5j')
    p.get_modified_residues('4v5j,1cbs')

    p.get_mutated_residues('1bgj')
    p.get_mutated_residues('1bgj,4v5j')

    p.get_release_status('1cbs')
    p.get_release_status('1cbs,4v5j')

    p.get_observed_ranges('1cbs')
    p.get_observed_ranges('1cbs,4v5j')

    p.get_observed_ranges_in_pdb_chain('1cbs', "A")

    p.get_secondary_structure('1cbs')
    p.get_secondary_structure('1cbs,4v5j')

    p.get_residue_listing('1cbs')

    p.get_residue_listing_in_pdb_chain('1cbs', "A")

    p.get_binding_sites('1cbs')
    p.get_binding_sites('1cbs,4v5j')

    p.get_files('1cbs')
    p.get_files('1cbs,4v5j')

    p.get_observed_residues_ratio('1cbs')
    p.get_observed_residues_ratio('1cbs,4v5j')

    p.get_assembly('1cbs')
    p.get_assembly('1cbs,4v5j')

    p.get_electron_density_statistics('1cbs')
    p.get_electron_density_statistics('1cbs,4v5j')

    p.get_drugbank_annotation('5hht')
    p.get_drugbank_annotation('5hht,5hht')

    p.get_functional_annotation('1cbs,1cbs')
    p.get_functional_annotation('1cbs')

    p.get_related_dataset('5o8b')
    p.get_related_dataset('5o8b,5o8b')





from bioservices.wikipathway import  WikiPathways
import pytest
import os
from easydev import TempFile

skiptravis = pytest.mark.skipif( "TRAVIS_PYTHON_VERSION" in os.environ,
    reason="On travis")

# The skip are those that fail on travis (june 2017) related to pandas
@skiptravis
@pytest.fixture
def wikipath():
    return WikiPathways(verbose=False)

@skiptravis
def test_organism(wikipath):
    assert "Homo sapiens" in wikipath.organisms
    wikipath.organism = 'Homo sapiens'
    assert wikipath.organism == 'Homo sapiens'

    try:
        wikipath.organism = 'Homo sapi'
        assert False
    except ValueError:
        assert True

    assert len(wikipath.organisms) > 2 # check it is not empty

@skiptravis
def test_showPathwayInBrowser(wikipath):
    wikipath.showPathwayInBrowser("WP232")

@skiptravis
def test_listPathways(wikipath):
    l = wikipath.listPathways()
    try:
        # FIXME pandas needed on travis. to do in v2
        assert len(l) > 40
        l = wikipath.listPathways("Homo sapiens")
        assert len(l) > 40
    except:
        pass

@skiptravis
def test_getPathway(wikipath):
    wikipath.getPathway("WP232")

@skiptravis
def test_getPathwayInfo(wikipath):
    wikipath.getPathwayInfo("WP232")

@skiptravis
def test_getPathwayAs(wikipath):
    res = wikipath.getPathwayAs("WP4", filetype="png")

@skiptravis
def test_findPathwaysByText(wikipath):
    res = wikipath.findPathwaysByText(query="p53")
    res = wikipath.findPathwaysByText(query="p53", species='Homo sapiens')
    assert len(wikipath.findPathwaysByText(query="p53 OR mapk",species='Homo sapiens'))>0

@skiptravis
def test_getOntologyTersmByPathway(wikipath):
    res = wikipath.getOntologyTermsByPathway("WP4")

@skiptravis
def _test_getCurationTags(wikipath):
    wikipath.getCurationTags("WP4")

@skiptravis
def _test_getcurationTagByNames(wikipath):
    wikipath.getCurationTagsByName("Curation:Tutorial")

@skiptravis
def test_findInteractions(wikipath):
    try:
        # FIXME pandas needed on travis. to do in v2
        assert len(wikipath.findInteractions("P53").species) > 10
    except:
        pass

@skiptravis
def test_getRecentChanges(wikipath):
    wikipath.getRecentChanges(20120101000000)

# does not seem to work
@skiptravis
def test_findPathwayByXref(wikipath):
    df = wikipath.findPathwaysByXref('P45985')
    try:
        # FIXME need Pandas on travis. will be fixed in v2
        assert len(df)
        assert df['x.id'].unique() == ['P45985']
    except:
        pass


@skiptravis
def test_findPathwaysByLitterature(wikipath):
    wikipath.findPathwaysByLiterature(18651794)


@skiptravis
def test_savePathwayAs(wikipath):
    # Note that not all WP have the PDF format available.
    # WP4 has not (march 2018)
    with TempFile(suffix=".png") as fout:
        wikipath.savePathwayAs("WP232", fout.name, display=False)


@skiptravis
def test_getPathwaysByParentOntologyTerm(wikipath):
    wikipath.getPathwaysByParentOntologyTerm("DOID:344")


@skiptravis
def test_createPathway(wikipath):
    try:
        wikipath.createPathway("","")
        assert False
    except NotImplementedError:
        assert True

@skiptravis
def test_updatePathwa(wikipath):
    try:
        wikipath.updatePathway("","","")
        assert False
    except NotImplementedError:
        assert True

@skiptravis
def test_saveCurationTag(wikipath):
    try:
        wikipath.saveCurationTag("","","")
        assert False
    except NotImplementedError:
        assert True

@skiptravis
def test_login(wikipath):
    try:
        wikipath.login("dummy", "dummy")
        assert False
    except NotImplementedError:
        assert True
@skiptravis
def test_remoceCurationTag(wikipath):
    try:
        wikipath.removeCurationTag("dummy", "dummy")
        assert False
    except NotImplementedError:
        assert True

# FIXME SLOW
def _test_getPathwayHistory(wikipath):
    _ = wikipath.getPathwayHistory("WP455", "20100101000000")

def test_coloredPathway(wikipath):
    wikipath.getColoredPathway("WP4",revision=0)






from bioservices import KEGG, KEGGParser
import pytest


@pytest.fixture(scope = "module")
def kegg():
    k = KEGG()
    k.organismIds
    k.organism = "hsa"
    return k


@pytest.fixture(params = [None, "", 0, 400, {}])
def parse_input(request):
    yield request.param


# This is a simple test class that do not test everything on purpose.
# The other class could be use to test the code more thoroughly but it takes several
# minutes so during development this one should be used instead.
# class TestKEGGAll should serve as a complement to this class

def test_get_pathway_by_gene(kegg):
    res = kegg.get_pathway_by_gene("7535", "hsa")
    assert isinstance(res, dict) is True
    assert 'hsa04064' in res.keys()

def test_isOrganism(kegg):
    assert kegg.isOrganism('T01440') == True
    assert kegg.isOrganism('hsa') == True
    assert kegg.isOrganism('dummy') == False

def test_database_IDs(kegg):
    kegg.pathwayIds

def test_conv(kegg):
    kegg.conv("ncbi-gi","hsa:10458+ece:Z5100")

def test_info(kegg):
    kegg.dbinfo("kegg")
    kegg.dbinfo("brite")

def test_list(kegg):
    kegg.list("pathway", "hsa")      # returns the list of human pathways

def test_find(kegg):
    kegg.find("compound", "300-310", "mol_weight")

def test_get(kegg):
    kegg.get("cpd:C01290+gl:G00092")

def test_checkDB(kegg):
    for this in ["info", "list", "find", "link"]:
        try:
            kegg._checkDB("dummy", this)
            assert False
        except:
            assert True
        kegg._checkDB("pathway", this)

def test_link(kegg):
    kegg.link("pathway", "hsa:10458+ece:Z5100")

def test_org_conv(kegg):
    assert 'hsa' == kegg.Tnumber2code("T01001")
    assert 'T01001' == kegg.code2Tnumber("hsa")

def test_parse_kgml_pathway(kegg):
    res = kegg.parse_kgml_pathway("hsa04660")

@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ids1(kegg):
    assert kegg.enzymeIds[0].startswith("ec")
@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ids2(kegg):
    assert kegg.compoundIds[0].startswith("cpd")
@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ids3(kegg):
    assert kegg.glycanIds[0].startswith("gl")
@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ids4(kegg):
    assert kegg.reactionIds[0].startswith("rn")
@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ids5(kegg):
    assert kegg.drugIds[0].startswith("dr")
@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ids6(kegg):
    assert kegg.koIds[0].startswith("ko")
@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ids7(kegg):
    assert kegg.briteIds[0].startswith("br")


def test_lookfor(kegg):
    kegg.lookfor_organism("human")
    kegg.lookfor_pathway("cell")

def test_organism(kegg):
    kegg.organism = "hsa"
    try:
        kegg.organism = "dummy"
        assert False
    except:
        assert True

def test_pathwayIDs(kegg):
    kegg.organism = "hsa"
    kegg.pathwayIds

def test_info(kegg):
    kegg.dbinfo("hsa")
    try:
        kegg.dbinfo("dummy")
        assert False
    except:
        assert True

def test_list(kegg):
    kegg.list("pathway") # returns the list of reference pathways
    kegg.list("organism") # returns the list of KEGG organisms with taxonomic classification
    kegg.list("hsa") # returns the entire list of human genes
    kegg.list("T01001") # same as above
    kegg.list("hsa:10458+ece:Z5100") # returns the list of a human gene and an E.coli O157 gene
    kegg.list("cpd:C01290+gl:G00092") # returns the list of a compound entry and a glycan entry
    kegg.list("C01290+G00092") # same as above

    # invalid queries:
    try:
        kegg.list("drug", "hsa")
        assert False
    except:
        assert True

    try:
        kegg.list("dumy")
        assert False
    except:
        assert True

def test_find(kegg):
    kegg.find("genes", "shiga+toxin") # for keywords "shiga" and "toxin"
    kegg.find("genes", "shiga toxin") # for keywords "shigatoxin"
    kegg.find("compound", "C7H10O5", "formula") # for chemicalformula "C7H10O5"
    kegg.find("compound", "O5C7","formula") # for chemicalformula containing "O5" and "C7"
    kegg.find("compound", "174.05","exact_mass") # for 174.045 =<exact mass < 174.055
    kegg.find("compound", "300-310","mol_weight") # for 300 =<molecular weight =< 310

def test_get(kegg):
    kegg.get("C01290+G00092")
    kegg.get("hsa:10458+ece:Z5100")
    kegg.get("hsa:10458+ece:Z5100", "aaseq")
    res = kegg.get("hsa05130", "image")
    try:
        kegg.get("hsa05130", "imagffe")
        assert False
    except:
        assert True


def test_parse(kegg, parse_input):
    # Check that parse can handle return values that get()
    # might reasonably produce like Service.response_codes
    assert isinstance(kegg.parse(parse_input), dict)


def test_conv(kegg):
    kegg.conv("ncbi-gi","hsa:10458+ece:Z5100")

    try:
        kegg.conv("unipro", "hsa")
        assert False
    except:
        assert True

    try:
        kegg.conv("uniprot", "hs")
        assert False
    except:
        assert True

    try:
        kegg.conv("hs", "unipro")
        assert False
    except:
        assert True

    try:
        kegg.conv("hsa", "unipr")
        assert False
    except:
        assert True

    # asc contains 1500. Try to get even samller to spped up tests.
    #kegg.conv("asc", "uniprot")
    kegg.conv("hsa","up:Q9BV86+")


def test_show_module(kegg):
    kegg.show_module("md:hsa_M00001")


def test_show_pathway(kegg):
    kegg.show_entry("path:hsa05416")
    kegg.show_pathway("path:hsa05416", scale=50)


def pathway2sif(kegg):
    sif = kegg.pathway2sif("path:hsa05416")


def test_KEGGParser(kegg):
    s = kegg
    d = s.parse(s.get("cpd:C00001"))
    d = s.parse(s.get("ds:H00001"))
    d = s.parse(s.get("dr:D00001"))
    d = s.parse(s.get("ev:E00001"))
    d = s.parse(s.get("ec:1.1.1.1"))
    d = s.parse(s.get("hsa:1525"))
    d = s.parse(s.get("genome:T00001"))
    d = s.parse(s.get("gl:G00001"))
    d = s.parse(s.get("md:hsa_M00554"))
    d = s.parse(s.get("ko:K00001"))
    d = s.parse(s.get("path:hsa04914"))
    d = s.parse(s.get("rc:RC00001"))
    d = s.parse(s.get("rn:R00001"))
    d = s.parse(s.get("rp:RP00001"))


    d = s.parse(s.get('C15682'))
    assert d['SEQUENCE'][0]['TYPE'] == 'PK'
    assert d['SEQUENCE'][0]['GENE'] =="0-2 mycAI [UP:Q83WF0]; 3 mycAII [UP:Q83WE9]; 4-5 mycAIII [UP:Q83WE8]; 6 mycAIV [UP:Q83WE7]; 7 mycAV [UP:Q83WE6]"
    assert d['SEQUENCE'][0]['ORGANISM'] == "Micromonospora griseorubida"


    #issue #79

    d = s.parse(s.get("C00395"))
    assert d["SEQUENCE"][0]["GENE"] == '[1] 0-2 pcbAB [UP:P19787] [2] 0-2 pcbAB [UP:P27742]'
    assert d["SEQUENCE"][0]["ORGANISM"] == '[1] Penicillium chrysogenum [2] Emericella nidulans (Aspergillus nidulans [GN:ani] )'
    assert d['SEQUENCE'][0]['SEQUENCE'] == '0 Aad  1 Cys  2 Val'
    assert d['SEQUENCE'][0]['TYPE'] == "NRP"


def test_KEGGParser_parse_invalid(parse_input):
    kp = KEGGParser()

    # Check that an exception is raised for invalid input
    with pytest.raises(ValueError):
        kp.parse(parse_input)



from bioservices.quickgo import QuickGO
import pytest

@pytest.fixture
def quickgo():
    return QuickGO(verbose=False, cache=False)

def test_annotation(quickgo):
    res = quickgo.Annotation(taxonId='9606')
    res = quickgo.Annotation(taxonId="9606", limit=100, aspect="F")
    res = quickgo.Annotation(taxonId="9606", limit=100, geneProductType="protein")
    res = quickgo.Annotation(taxonId="9606", limit=100, geneProductType="protein", assignedBy="UniProt")
    res = quickgo.Annotation(taxonId="9606", evidenceCode="ECO:0000501")






def test_annotations_from_goid(quickgo):
    df = quickgo.Annotation_from_goid(goId="GO:0003824,GO:0003677", taxonId="9606" )
    assert len(df)

def test_Term(quickgo):
    res = quickgo.get_go_terms("GO:0003824,GO:0003677")
    assert len(res) == 2

def test_gosearch(quickgo):
    res = quickgo.go_search("GO:0022804")
    assert len(res)

def test_go_ancestors(quickgo):
    res = quickgo.get_go_ancestors("GO:0022804")
    assert len(res)

def test_go_children(quickgo):
    res = quickgo.get_go_children("GO:0022804")
    assert len(res)

def test_go_chart(quickgo):
    res = quickgo.get_go_chart("GO:0022804")

def test_go_paths(quickgo):
    res = quickgo.get_go_paths("GO:0022804", "GO:0005215")
    assert len(res)




from bioservices.muscle import MUSCLE
from bioservices import uniprot
import pytest
import os

skiptravis = pytest.mark.skipif( "TRAVIS_PYTHON_VERSION" in os.environ,
     reason="too long On travis ")

@skiptravis
@pytest.mark.xfail
def test_muscle():
    m = MUSCLE(verbose=False)
    m.parameters
    m.get_parameter_details("format")
    try:
        m.get_parameter_details("formattt")
        assert False
    except:
        assert True

    u = uniprot.UniProt(verbose=False)
    f1 = u.get_fasta("P18812")
    f2 = u.get_fasta("P18813")

    jobid = m.run(frmt="fasta", sequence=f1+f2, email="cokelaer@ebi.ac.uk")
    m.get_status(jobid)
    m.wait(jobid)

    m.get_result_types(jobid)
    m.get_result(jobid, 'phylotree')






from bioservices import Ensembl
import pytest

@pytest.fixture
def ensembl():
    return Ensembl(verbose=False)


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_get_archive(ensembl):
    res = ensembl.get_archive("ENSG00000157764")
    assert 'id' in res.keys()


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_post_archive(ensembl):
    res = ensembl.post_archive(["ENSG00000157764"])
    assert "ENSG00000157764" in [x['id'] for x in res]


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_genetree_by_id(ensembl):
    res = ensembl.get_genetree_by_id("ENSGT00390000003602",
                                    frmt='phyloxml', aligned=False, sequence='cdna')[0:1000]
    assert "taxonomy" in res
    assert "117571" in res
    assert "Euteleos" in res

    ensembl.get_genetree_by_id('ENSGT00390000003602', frmt='nh',
                              nh_format='simple')
    ensembl.get_genetree_by_id('ENSGT00390000003602', frmt='phyloxml')
    ensembl.get_genetree_by_id('ENSGT00390000003602', frmt='phyloxml',
                              aligned=True, sequence='cdna')
    ensembl.get_genetree_by_id('ENSGT00390000003602', frmt='phyloxml',
                              sequence='none')


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_get_genetree_by_member_id(ensembl):
    res = ensembl.get_genetree_by_member_id('ENSG00000157764',
                                           frmt='json', nh_format='phylip')
    assert len(res) > 1000


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_get_genetree_by_member_symbol(ensembl):
    res = ensembl.get_genetree_by_member_symbol('human', 'BRCA2',
                                               nh_format='simple')
    # was working until dec 2014 then results changed. so not stable
    # assert res[0:200] == """((((((((ENSPFOP00000001575:0.046083,ENSXMAP00000006983:0.065551):0.43822,ENSONIP00000006940:0.359035):0.019582,((ENSTRUP00000015030:0.077336,ENSTNIP00000002435:0.099898):0.208834,ENSGACP00000015199:0."""


# FIXME
# feb 2020. does not work even on ensemble website
def __test_get_alignment_by_region(ensembl):
    region = '2:106040000-106040050'
    species = 'taeniopygia_guttata'
    res = ensembl.get_alignment_by_region(region, species,
                                         species_set_group='sauropsids')

    assert 'gallus_gallus' in res[0]['tree']


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_get_homology_by_id(ensembl):
    res = ensembl.get_homology_by_id('ENSG00000157764')
    assert res.keys()
    res = ensembl.get_homology_by_id('ENSG00000157764', frmt='xml')
    res = ensembl.get_homology_by_id('ENSG00000157764', format='condensed',
                                    type='orthologues', target_taxon='10090')
    assert 'homologies' in res['data'][0].keys()


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_references(ensembl):
    # does not work anymore Dev 2018 returns empty list
    #res = ensembl.get_xrefs_by_id('ENST00000288602', external_db='PDB',
    #                             all_levels=True)
    #assert res[0]['dbname'] == 'PDB'
    res = ensembl.get_xrefs_by_id('ENST00000288602')
    assert len(res)

    res = ensembl.get_xrefs_by_name('BRCA2', 'human')
    assert 'db_display_name' in res[0].keys()

    res = ensembl.get_xrefs_by_symbol('BRCA2', 'homo_sapiens',
                                     external_db='HGNC')
    assert 'id' in res[0]


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_info(ensembl):
    assert len(ensembl.get_info_analysis('human')) > 0
    assert len(ensembl.get_info_assembly('human')['karyotype']) == 25

    ensembl.get_info_assembly_by_region('homo_sapiens', 'X')

    assert len(ensembl.get_info_biotypes('human'))

    assert ensembl.get_info_compara_methods()

    assert ensembl.get_info_compara_by_method('EPO')[0]['method'] == 'EPO'

    assert ensembl.get_info_comparas()['comparas']

    assert ensembl.get_info_data()

    res = ensembl.get_info_external_dbs('human')
    assert 'HGNC' in [x['name'] for x in res if 'hgnc' in x['name'].lower()]

    assert ensembl.get_info_ping()
    assert ensembl.get_info_rest()
    assert ensembl.get_info_software()
    res = ensembl.get_info_species()
    assert 'ovis_aries' in [x['name'] for x in res['species'] if 'ovis' in x['name']]


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_lookup(ensembl):
    res = ensembl.get_lookup_by_id('ENSG00000157764', expand=True)

    res = ensembl.get_lookup_by_id('ENSG00000157764', expand=True)
    assert res.keys()

    res = ensembl.post_lookup_by_id(["ENSG00000157764", "ENSG00000248378"],
                                   expand=0)

    res = ensembl.get_lookup_by_symbol('homo_sapiens', 'BRCA2', expand=True)
    assert len(res['Transcript'])

    res = ensembl.post_lookup_by_symbol('human', ["BRCA2", "BRAF"], expand=True)
    assert len(res['BRCA2']['Transcript'])


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_mapping(ensembl):
    res = ensembl.get_map_assembly_one_to_two('GRCh37', 'GRCh38',
                                             region='X:1000000..1000100:1')

    res = ensembl.get_map_translation_to_region('ENSP00000288602', '100..300')
    assert res['mappings'][0]

    res = ensembl.get_map_cds_to_region('ENST00000288602', '1..1000')

    res = ensembl.get_map_cdna_to_region('ENST00000288602', '100..300')


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_ontologies(ensembl):
    res = ensembl.get_ontology_ancestors_by_id('GO:0005667')
    res = ensembl.get_ontology_ancestors_chart_by_id('GO:0005667')
    res = ensembl.get_ontology_descendants_by_id('GO:0005667')
    res = ensembl.get_ontology_by_id('GO:0005667')
    assert res['accession']

    res = ensembl.get_ontology_by_name('transcription factor complex')
    res = ensembl.get_taxonomy_classification_by_id(9606)
    assert res[0]['children']
    res = ensembl.get_ontology_by_id('GO:0005667')
    assert 'children' in res.keys()
    res = ensembl.get_ontology_by_name('transcription factor complex')
    assert res[0]['children']
    res = ensembl.get_ontology_by_name('transcription factor')
    assert res


@pytest.mark.flaky(max_runs=3, min_passes=1)
def test_taxonomy(ensembl):
    res = ensembl.get_taxonomy_by_name('homo')
    assert len(res[0]) > 1

    res = ensembl.get_taxonomy_classification_by_id('9606')

    res = ensembl.get_taxonomy_by_name('Homo')

    ensembl.get_taxonomy_by_id(9606)['scientific_name']

    res = ensembl.get_overlap_by_id("ENSG00000157764", feature='gene')

    res = ensembl.get_overlap_by_region('7:140424943-140624564',
                                       species='human', feature='gene')

    res = ensembl.get_overlap_by_translation('ENSP00000288602',
                                            type='Superfamily')

    res = ensembl.get_overlap_by_translation('ENSP00000288602',
                                            type='missense_variant',
                                            feature='transcript_variation')

    res = ensembl.get_overlap_by_translation('ENSP00000288602',
                                            type='missense_variant',
                                            feature='somatic_transcript_variation')

def _test_regulation(ensembl):
    res = ensembl.get_regulatory_by_id('ENSR00001348195', 'human')
    assert 'ID' in res[0].keys()


def test_sequences(ensembl):
    sequence = ensembl.get_sequence_by_id('ENSG00000157764', frmt='text')
    #assert sequence.startswith("CGCCTCCCTTCCCCCTCCCCGCCCGACAGCGGCCGCTCGGGCCCCG")
    # changed June 2018. double checked on web site
    # http://www.ensembl.org/Homo_sapiens/Gene/Sequence?g=ENSG00000157764;r=7:140719327-140924928
    # fails on Sep 2002
    #assert sequence.startswith("TTCCCCCAATCCCCTCAGGCTCGGCTGCGCCCGGGGCCGCGGGCCGGTACCTGAGGTGGC")

    #FIXME fails on travis
    #sequence = ensembl.get_sequence_by_id('ENSG00000157764', frmt='fasta')
    #assert sequence.startswith(">ENSG00000157764 chromosome:")

    sequence = ensembl.get_sequence_by_id('CCDS5863.1', frmt='fasta',
                                         object_type='transcript', db_type='otherfeatures',
                                         type='cds', species='human')

    sequence = ensembl.get_sequence_by_id('ENSG00000157764', frmt='seqxml',
                                         multiple_sequences=True, type='protein')
    sequence = ensembl.get_sequence_by_region('X:1000000..1000100:1', 'human')
    assert sequence['id'] == 'chromosome:GRCh38:X:1000000:1000100:1'
    sequence = ensembl.get_sequence_by_region('ABBA01004489.1:1..100', 'human',
                                             frmt='json', coord_system='seqlevel')

    seq = ensembl.get_sequence_by_id('ENSE00001154485', expand_5prime=10,
                                     frmt='fasta', type='genomic')
    assert seq == u'>ENSE00001154485.4 chromosome:GRCh38:7:140924566:140924752:-1\nCCCTCCCCGCCCGACAGCGGCCGCTCGGGCCCCGGCTCTCGGTTATAAGATGGCGGCGCT\nGAGCGGTGGCGGTGGTGGCGGCGCGGAGCCGGGCCAGGCTCTGTTCAACGGGGACATGGA\nGCCCGAGGCCGGCGCCGGCGCCGGCGCCGCGGCCTCTTCGGCTGCGGACCCTGCCATTCC\nGGAGGAG\n'
    res = ensembl.get_sequence_by_id('ENSG00000157764', frmt='seqxml',
                                    type='protein', multiple_sequences=1)
    assert 'MAAL' in res

    res = ensembl.get_sequence_by_id('GENSCAN00000000001', frmt='json',
                                    object_type='predictiontranscript', db_type='core',
                                    species='homo_sapiens', type='protein')
    assert 'MERGKK' in res['seq']

    res = ensembl.get_sequence_by_id('ENSP00000288602', frmt='json')
    assert res['seq'].startswith("MAAL")


def test_variation(ensembl):
    res = ensembl.get_variation_by_id('rs56116432', 'human')
    assert 'MAF' in res.keys()
    res = ensembl.get_vep_by_id('COSM476', 'human')
    res = ensembl.get_vep_by_id('rs116035550', 'human')
    res = ensembl.get_vep_by_region('9:22125503-22125502:1', 'C', 'human')
    assert res[0]['most_severe_consequence']

    ensembl.get_variation_by_id("rs56116432", species='homo_sapiens')
    ensembl.get_variation_by_id("rs56116432", species='homo_sapiens',
                               pops=1, genotypes=1, phenotypes=1)



import pytest

from bioservices.omicsdi import OmicsDI

@pytest.fixture
def omicsdi():
    return OmicsDI(verbose=True)

def keys_exists(dict_, keys, test = all):
    return test(key in dict_ for key in keys)

def test_dataset_merge(omicsdi):
    with pytest.raises(Exception):
        omicsdi.dataset_merge()

def test_dataset_get_merge_candidates(omicsdi):
    with pytest.raises(Exception):
        omicsdi.dataset_get_merge_candidates()

def test_dataset_domain_accession_files(omicsdi):
    response = omicsdi.dataset_domain_accession_files(
        accession = "PXD000210", domain = "pride", position = 1)

    assert len(response)
    assert keys_exists(response[0], ("type", "url"))

def test_dataset_search(omicsdi):
    response = omicsdi.dataset_search()
    assert keys_exists(response, ("count", "datasets"))

def test_dataset_latest(omicsdi):
    response = omicsdi.dataset_latest()
    assert keys_exists(response, ("count", "datasets"))

def test_dataset_batch(omicsdi):
    response = omicsdi.dataset_batch(accession = "PXD000210", database = "pride")
    assert keys_exists(response, ("failure", "datasets"))

def test_dataset_most_accessed(omicsdi):
    response = omicsdi.dataset_most_accessed(size = 20)
    assert keys_exists(response, ("count", "datasets"))

def test_dataset_get_file_links(omicsdi):
    response = omicsdi.dataset_get_file_links(accession = "PXD000210", database = "pride")
    assert len(response)

def test_dataset_domain_accession(omicsdi):
    response = omicsdi.dataset_domain_accession(accession = "PXD000210", domain = "pride")
    assert keys_exists(response, ("database", "file_versions"))

def test_dataset_get_similar(omicsdi):
    response = omicsdi.dataset_get_similar(accession = "PXD000210", database = "pride")
    assert keys_exists(response, ("count", "datasets"))

def test_dataset_get_similar_by_pubmed(omicsdi):
    response = omicsdi.dataset_get_similar_by_pubmed(pubmed = 16585740)
    assert len(response)
    assert keys_exists(response[0], ("accession", "database", "initHashCode"), test = any)

def test_database_all(omicsdi):
    response = omicsdi.database_all()
    assert len(response)

def test_term_get_term_by_pattern(omicsdi):
    response = omicsdi.term_get_term_by_pattern()
    assert keys_exists(response, ("total_count", "items"))

    response = omicsdi.term_get_term_by_pattern(query = "array", size = 10)
    assert len(response["items"])

def test_term_frequently_term_list(omicsdi):
    response = omicsdi.term_frequently_term_list(domain = "pride", field = "description")
    assert len(response)
    assert keys_exists(response[0], ("label", "frequent"))

def test_seo_home(omicsdi):
    response = omicsdi.seo_home()
    assert keys_exists(response, ("@graph",))

def test_seo_search(omicsdi):
    response = omicsdi.seo_search()
    assert keys_exists(response, ("name", "url", "keywords",
        "description", "variableMeasured", "alternateName",
        "logo", "email", "image", "potentialAction",
        "sameAs", "creator", "citation", "primaryImageOfPage",
        "@context", "@type"))

def test_seo_api(omicsdi):
    response = omicsdi.seo_api()
    assert keys_exists(response, ("name", "url", "keywords",
        "description", "variableMeasured", "alternateName",
        "logo", "email", "image", "potentialAction",
        "sameAs", "creator", "citation", "primaryImageOfPage",
        "@context", "@type"))

def test_seo_database(omicsdi):
    response = omicsdi.seo_database()
    assert keys_exists(response, ("name", "url", "keywords",
        "description", "variableMeasured", "alternateName",
        "logo", "email", "image", "potentialAction",
        "sameAs", "creator", "citation", "primaryImageOfPage",
        "@context", "@type"))

def test_seo_dataset_domain_accession(omicsdi):
    response = omicsdi.seo_dataset_domain_accession(domain = "pride", accession = "PXD000210")
    assert keys_exists(response, ("name", "url", "keywords"), test = any)

def test_seo_about(omicsdi):
    response = omicsdi.seo_about()
    assert keys_exists(response, ("name", "url", "keywords",
        "description", "variableMeasured", "alternateName",
        "logo", "email", "image", "potentialAction",
        "sameAs", "creator", "citation", "primaryImageOfPage"))

def test_statistics_organisms(omicsdi):
    response = omicsdi.statistics_organisms()
    assert len(response)
    assert keys_exists(response[0], ("label", "id", "value", "name"))

    response = omicsdi.statistics_organisms(size = 0)
    assert len(response)

def test_statistics_tissues(omicsdi):
    response = omicsdi.statistics_tissues()
    assert len(response)
    assert keys_exists(response[0], ("label", "id", "value", "name"))

    response = omicsdi.statistics_tissues(size = 0)
    assert len(response)

def test_statistics_omics(omicsdi):
    response = omicsdi.statistics_omics()
    assert len(response)
    assert keys_exists(response[0], ("label", "id", "value", "name"))

def test_statistics_diseases(omicsdi):
    response = omicsdi.statistics_diseases()
    assert len(response)
    assert keys_exists(response[0], ("label", "id", "value", "name"))

def test_statistics_domains(omicsdi):
    response = omicsdi.statistics_domains()
    assert len(response)
    assert keys_exists(response[0], ("domain", "subdomains"))

def test_statistics_omics_by_year(omicsdi):
    response = omicsdi.statistics_omics_by_year()
    assert len(response)
    assert keys_exists(response[0], ("year", "genomics",
        "metabolomics", "proteomics", "transcriptomics"))


from bioservices import PSICQUIC, psicquic
import pytest

@pytest.fixture
def psicquic():
    return PSICQUIC(verbose=False)


def test_read_registry(psicquic):
    psicquic.read_registry()

def test_print_status(psicquic):
    psicquic.print_status()

def test_registry(psicquic):
    N1 = len(psicquic.registry_names)
    N2 = len(psicquic.registry_versions)
    N2 = len(psicquic.registry_restricted)
    assert N1 == N2

def test_query(psicquic):
    if 'intact' not in psicquic.activeDBs or 'matrixdb' not in psicquic.activeDBs:
        return

    psicquic.query("intact", "brca2", "tab27")
    psicquic.query("intact", "zap70", "xml25")
    psicquic.query("matrixdb", "*", "xml25")
    try:
        psicquic.query("matxdb", "*", "xml25")
        assert False
    except:
        assert True

    psicquic.query("matrixdb", "*", "xml25", firstResult=10, maxResults=10)

    # accessing the string DB
    if 'string' in psicquic.activeDBs:
        psicquic.query("string", "species:10090", firstResult=0, maxResults=100, output="tab25")

def _test_appsPPI():
    p = psicquic.AppsPPI(verbose=False)
    p.queryAll("ZAP70", ["intact"])
    p.summary()




from bioservices.mygeneinfo import MyGeneInfo

mgi = MyGeneInfo()


def test_get_all_genes():
    res =  mgi.get_genes("301345,22637")
    assert len(res) == 2
    assert res[0]['_id'] == '301345'


    mgi.get_genes(("301345,22637"))
    # first one is rat, second is mouse. This will return a 'notfound'
    # entry and the second entry as expected.
    res = mgi.get_genes("301345,22637", species="mouse") 
    assert '_id' not in res[0]
    assert res[1]['_id'] == '22637'
    assert res[1]['taxid'] == 10090


def test_get_one_gene():
    res =  mgi.get_one_gene("301345")
    assert res['_id'] == '301345'
    assert res['taxid'] == 10116


def test_get_one_query():
    res = mgi.get_one_query("zap70", size=10, dotfield=True, sort="taxid")
def test_get_queries():
    res = mgi.get_queries("zap70,zap70", dotfield=True)


def test_get_metadata():   
    res = mgi.get_metadata()
    res = mgi.get_taxonomy()
    assert 'human' in res



import os.path as osp

import pytest
from easydev import TempFile

from bioservices.bigg import BiGG

@pytest.fixture
def bigg():
    return BiGG(verbose=True)

def keys_exists(dict_, keys, test = all):
    return test(key in dict_ for key in keys)

def test_version(bigg):
    version = bigg.version
    assert keys_exists(version,
        ("bigg_models_version", "api_version", "last_updated"))

    assert version["api_version"] == bigg._api_version

def test_length(bigg):
    assert len(bigg)

def test_models(bigg):
    models = bigg.models
    assert isinstance(models, list)

    assert keys_exists(models[0],
        ("bigg_id", "metabolite_count", "organism", "reaction_count",
        "gene_count"))

def test_metabolites(bigg):
    metabolites = bigg.metabolites("iND750")
    assert isinstance(metabolites, list)

    assert keys_exists(metabolites[0],
        ("bigg_id", "model_bigg_id", "organism", "name", "compartment_bigg_id"))

    metabolite = bigg.metabolites("iND750", "10fthf_c")
    assert keys_exists(metabolite,
        ("formula", "bigg_id", "compartment_bigg_id", "name", "model_bigg_id",
        "old_identifiers", "escher_maps", "other_models_with_metabolite",
        "database_links"))

    metabolites = bigg.metabolites("iND750", ids=["10fthf", "12dgr_SC"])
    assert isinstance(metabolites, list)
    assert len(metabolites)

def test_reactions(bigg):
    reactions = bigg.reactions("iND750")
    assert isinstance(reactions, list)

    assert keys_exists(reactions[0],
        ("bigg_id", "model_bigg_id", "organism", "name"))

    reaction = bigg.reactions("iND750", "GAPD")
    assert keys_exists(reaction,
        ("name", "pseudoreaction", "count", "model_bigg_id", "metabolites",
        "database_links", "escher_maps", "other_models_with_reaction",
        "count", "results"))

    reactions = bigg.reactions("iND750", ids=["13BGH", "13BGHe"])
    assert isinstance(reactions, list)
    assert len(reactions)

def test_genes(bigg):
    genes = bigg.genes("iMM904")
    assert isinstance(genes, list)

    assert keys_exists(genes[0],
        ("bigg_id", "model_bigg_id", "organism", "name"))

    gene = bigg.genes("iMM904", "Q0045")
    assert keys_exists(gene,
        ("bigg_id", "name", "strand", "leftpos", "genome_ref_string",
        "database_links", "rightpos", "protein_sequence", "model_bigg_id",
        "reactions", "old_identifiers", "mapped_to_genbank",
        "genome_name", "chromosome_ncbi_accession"))

    genes = bigg.genes("iMM904", ids=["Q0045", "Q0080"])
    assert isinstance(genes, list)
    assert len(genes)

def test_universal(bigg):
    for type_ in ("metabolites", "reactions"):
        results = getattr(bigg, type_)()
        assert isinstance(results, list)

        result = results[0]
        assert keys_exists(result,
            ("bigg_id", "model_bigg_id", "name"))

        assert result["model_bigg_id"] == "Universal"

def test_search(bigg):
    with pytest.raises(TypeError):
        bigg.search("foobar", "foobar")
    
    models = bigg.search("e coli", "models")
    assert keys_exists(models[0],
        ("bigg_id", "organism", "metabolite_count", "gene_count",
        "reaction_count"))

    required_keys = ("model_bigg_id", "bigg_id", "name", "organism")

    metabolites = bigg.search("g3p", "metabolites")
    assert keys_exists(metabolites[0], required_keys)

    genes = bigg.search("gap", "genes")
    assert keys_exists(genes[0], required_keys)

    reactions = bigg.search("phosphate", "reactions")
    assert keys_exists(reactions[0], required_keys)

def test_download(bigg):
    with pytest.raises(TypeError):
        bigg.download("iND750", format_="foobar")

    with TempFile() as f:
        bigg.download("iND750", target=f.name)

        assert osp.exists(f.name)




from bioservices import UniChem
import pytest

@pytest.fixture
def unichem():
    u = UniChem(verbose=False)
    return u


def test_get_compound_ids_from_src_id(unichem):
    res1 = unichem.get_compound_ids_from_src_id("CHEMBL2", "chembl", "chebi")
    res2 = unichem.get_compound_ids_from_src_id(["CHEMBL2"], "chembl", "chebi")
    assert res1 == res2[0]
    assert res1 == [{u'src_compound_id': u'8364'}]
    assert res2[0] == [{u'src_compound_id': u'8364'}]

def test_get_all_src_ids(unichem):
    assert len(unichem.get_all_src_ids())>=23

def test_get_source_id(unichem):
    assert unichem._get_source_id("chembl") == 1
    assert unichem._get_source_id("1") == 1
    assert unichem._get_source_id(1) == 1

    try:
        unichem._get_source_id("wrong")
        assert False
    except:
        assert True

    try:
        unichem._get_source_id("20000")
        assert False
    except:
        assert True

def test_get_src_information(unichem):
    assert unichem.get_source_information("chebi")['name'] == "chebi"

    assert unichem.get_source_information(['chembl', 'drugbank'])[0]['name']=="chembl"

def test_get_all_compound_ids_from_src_id(unichem):
    res = unichem.get_all_compound_ids_from_all_src_id("CHEMBL12", "chembl")
    res = unichem.get_all_compound_ids_from_all_src_id("CHEMBL12", "chembl", "chebi")

def test_mapping(unichem):
    res = unichem.get_mapping("kegg_ligand", "chembl")
    assert len(res)>0

def test_get_src_compound_ids_from_inchikey(unichem):
    unichem.get_src_compound_ids_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")
    unichem.get_src_compound_ids_from_inchikey(["AAOVKJBEBIDNHE-UHFFFAOYSA-N"])
    #unichem.get_src_compound_ids_all_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")
    #unichem.get_src_compound_ids_all_from_inchikey(["AAOVKJBEBIDNHE-UHFFFAOYSA-N"])

def test_structure(unichem):
    unichem.get_structure("CHEMBL12", "chembl")
    unichem.get_structure(["CHEMBL12"], "chembl")
    unichem.get_structure_all("CHEMBL12", "chembl")

def test_get_src_compound_id_url(unichem):
    unichem.get_src_compound_id_url("CHEMBL12", "chembl", "drugbank")
    unichem.get_src_compound_id_url(["CHEMBL12"], "chembl", "drugbank")

def test_get_src_compound_ids_all_from_obsolete(unichem):
    unichem.get_src_compound_ids_all_from_obsolete("DB07699", "2")[0]
    unichem.get_src_compound_ids_all_from_obsolete("DB07699", "2", "2")[0]

#SLOW one
def test_get_verbose_src_compound_ids_from_inchikey(unichem):
    assert unichem.get_verbose_src_compound_ids_from_inchikey("GZUITABIAKMVPG-UHFFFAOYSA-N") != 400
    assert unichem.get_verbose_src_compound_ids_from_inchikey("QFFGVLORLPOAEC-SNVBAGLBSA-N") != 400 


@pytest.mark.xfail
def test_get_auxiliary_mapping(unichem):
    # useless since we have a global timeout for pytest of 10 o r30 seconds
    unichem.settings.TIMEOUT = 100
    res = unichem.get_auxiliary_mappings(1)
    if isinstance(res, int):
        pass
    else:
        assert len(res)>0

